@@ -118,7 +118,7 @@ fi_to_py(struct fuse_file_info *fi)
 
 /* transform a Python integer to an unsigned C numeric value */
 
-#define py2attr(st, attr)						\
+#define py2attr(st, attr) {						\
 	if (PyInt_Check(pytmp) && sizeof((st)->attr) <= sizeof(long)) {	\
 		/*							\
 		 * We'd rather use here PyInt_AsUnsignedLong() here	\
@@ -162,8 +162,9 @@ fi_to_py(struct fuse_file_info *fi)
 		goto OUT_DECREF;					\
 	(st)->attr = ctmp;						\
 	if ((unsigned long long)(st)->attr != ctmp)			\
-		goto OUT_DECREF;
-		
+		goto OUT_DECREF;					\
+}
+
 #define fetchattr_nam(st, attr, aname)					\
 	if (!(pytmp = PyObject_GetAttrString(v, aname)))		\
 		goto OUT_DECREF;					\
@@ -174,7 +175,13 @@ fi_to_py(struct fuse_file_info *fi)
 
 #define fetchattr_soft(st, attr)					\
 	if (PyObject_HasAttrString(v, #attr)) {				\
-		fetchattr(st, attr);					\
+		pytmp = PyObject_GetAttrString(v, #attr);		\
+		if (!pytmp)						\
+			goto OUT_DECREF;				\
+	        if (pytmp == Py_None)					\
+			Py_DECREF(pytmp);				\
+		else							\
+			py2attr(st, attr);				\
 	}
 
 /*
@@ -671,8 +678,22 @@ getxattr_func(const char *path, const char *name, char *value, size_t size)
 #endif
 
 	if(PyString_Check(v)) {
-		if(PyString_Size(v) > size)
+        /* size zero can be passed into these calls  to return the current size of
+         * the named extended attribute
+         */
+        if (size == 0) {
+		    ret = PyString_Size(v);
 			goto OUT_DECREF;
+        } 
+
+        /* If the size of the value buffer is too small to hold the result,  errno
+         * is set to ERANGE.
+         */
+		if (PyString_Size(v) > size) {
+            ret = -ERANGE;
+			goto OUT_DECREF;
+        }
+
 		memcpy(value, PyString_AsString(v), PyString_Size(v));
 		ret = PyString_Size(v);
 	}
@@ -891,6 +912,7 @@ pyfuse_loop_mt(struct fuse *f)
 	save = PyEval_SaveThread();
 	err = fuse_loop_mt(f);
 	PyEval_RestoreThread(save);
+	interp = NULL;
 #endif
 
 	return(err);
